@@ -33,6 +33,7 @@ public:
 	void testECEVPSaveLoadStreamNoPass();
 	void testECEVPSaveLoadFile();
 	void testECEVPSaveLoadFileNoPass();
+	void testECEVPLoadKeyWrongPassword();
 
 	void testRSAEVPKeyFromX509();
 	void testRSAEVPKeyFromPKCS12();
@@ -41,6 +42,7 @@ public:
 	void testRSAEVPKeyByLength();
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
 	void testECEVPKeyByLength();
+	void testEVPKeyByModulus();
 #endif // OPENSSL_VERSION_NUMBER >= 0x30000000L
 #endif // OPENSSL_VERSION_NUMBER >= 0x10000000L
 
@@ -53,4 +55,4 @@ private:
 };
 
 
-#endif // EVPTest_INCLUDED
\ No newline at end of file
+#endif // EVPTest_INCLUDED
