@@ -4,7 +4,8 @@
 // Definition of the SQLExecutor class.
 //
 // Copyright (c) 2006, Applied Informatics Software Engineering GmbH.,
-// Aleph ONE Software Engineering d.o.o., and Contributors.
+// Aleph ONE Software Engineering LLC,
+// and Contributors.
 //
 // SPDX-License-Identifier:	BSL-1.0
 //
@@ -22,6 +23,8 @@
 #include "Poco/Data/RecordSet.h"
 #include "Poco/Exception.h"
 #include <iostream>
+#include <optional>
+#include <tuple>
 
 using namespace Poco::Data::Keywords;
 
@@ -66,7 +69,11 @@ public:
 		assertTrue (c.isConnected());
 		try
 		{
-			assertTrue (c.getTimeout() == 10);
+			int tout = c.getTimeout();
+			if (tout) // some drivers/DBMS (eg. postgres) do not cooperate here
+				assertEqual (10, tout);
+			else
+				std::cout << "Session timeout returned zero." << '\n';
 		}
 		catch(const NotSupportedException&)
 		{
@@ -75,7 +82,11 @@ public:
 
 		try
 		{
-			assertTrue (c.getLoginTimeout() == 10);
+			int tout = c.getLoginTimeout();
+			if (tout) // some drivers/DBMS (eg. postgres) do not cooperate here
+				assertEqual (10, tout);
+			else
+				std::cout << "Login timeout returned zero." << '\n';
 		}
 		catch(const NotSupportedException&)
 		{
@@ -360,6 +371,8 @@ public:
 		assertTrue(dd.isEmpty());
 	}
 	void nullable();
+	void stdOptional();
+	void stdTupleWithOptional();
 
 	void unicode(const std::string& dbConnString);
 	void encoding(const std::string& dbConnString);
@@ -372,6 +385,8 @@ public:
 private:
 	static const std::string MULTI_INSERT;
 	static const std::string MULTI_SELECT;
+	static const float EPSILON_FLOAT;
+	static const double EPSILON_DOUBLE;
 
 	Poco::Data::Session* _pSession;
 	Poco::Data::Session* _pEncSession;
