@@ -21,6 +21,7 @@
 #include "Poco/Data/Time.h"
 #include "Poco/Data/StatementImpl.h"
 #include "Poco/Data/RecordSet.h"
+#include "Poco/Data/SessionPool.h"
 #include "Poco/Data/Transaction.h"
 #include "Poco/Data/MySQL/Connector.h"
 #include "Poco/Data/MySQL/MySQLException.h"
@@ -29,6 +30,7 @@
 #include <Winsock2.h>
 #endif
 
+
 #include <mysql/mysql.h>
 #include <iostream>
 #include <limits>
@@ -143,7 +145,8 @@ private:
 
 SQLExecutor::SQLExecutor(const std::string& name, Poco::Data::Session* pSession):
 	CppUnit::TestCase(name),
-	_pSession(pSession)
+	_pSession(pSession),
+	_dataExecutor(name, pSession, nullptr, false)
 {
 }
 
@@ -156,14 +159,14 @@ SQLExecutor::~SQLExecutor()
 void SQLExecutor::bareboneMySQLTest(const char* host, const char* user, const char* pwd, const char* db, int port, const char* tableCreateString)
 {
 	int rc;
-	MYSQL* hsession = mysql_init(0);
-	assertTrue (hsession != 0);
+	MYSQL* hsession = mysql_init(nullptr);
+	assertTrue (hsession != nullptr);
 
-	MYSQL* tmp = mysql_real_connect(hsession, host, user, pwd, db, port, 0, 0);
+	MYSQL* tmp = mysql_real_connect(hsession, host, user, pwd, db, port, nullptr, 0);
 	assertTrue (tmp == hsession);
 
 	MYSQL_STMT* hstmt = mysql_stmt_init(hsession);
-	assertTrue (hstmt != 0);
+	assertTrue (hstmt != nullptr);
 
 	std::string sql = "DROP TABLE Test";
 	mysql_real_query(hsession, sql.c_str(), static_cast<unsigned long>(sql.length()));
@@ -183,7 +186,7 @@ void SQLExecutor::bareboneMySQLTest(const char* host, const char* user, const ch
 	int fourth = 4;
 	float fifth = 1.5;
 
-	MYSQL_BIND bind_param[5] = {{0}};
+	MYSQL_BIND bind_param[5] = {};
 
 	bind_param[0].buffer		= const_cast<char*>(str[0].c_str());
 	bind_param[0].buffer_length = static_cast<unsigned long>(str[0].length());
@@ -203,7 +206,12 @@ void SQLExecutor::bareboneMySQLTest(const char* host, const char* user, const ch
 	bind_param[4].buffer		= &fifth;
 	bind_param[4].buffer_type   = MYSQL_TYPE_FLOAT;
 
+#if LIBMYSQL_VERSION_ID >= 80300
+	rc = mysql_stmt_bind_named_param(hstmt, bind_param, 5, nullptr);
+#else
 	rc = mysql_stmt_bind_param(hstmt, bind_param);
+#endif
+
 	assertTrue (rc == 0);
 
 	rc = mysql_stmt_execute(hstmt);
@@ -218,7 +226,7 @@ void SQLExecutor::bareboneMySQLTest(const char* host, const char* user, const ch
 	fourth = 0;
 	fifth = 0.0f;
 
-	MYSQL_BIND bind_result[5] = {{0}};
+	MYSQL_BIND bind_result[5] = {};
 
 	bind_result[0].buffer		= chr[0];
 	bind_result[0].buffer_length = sizeof(chr[0]);
@@ -1371,7 +1379,7 @@ void SQLExecutor::timestamp()
 	std::string firstName("Simpson");
 	std::string address("Springfield");
 	DateTime birthday(1980, 4, 1, 5, 45, 12, 354, 879);
-	
+
 	int count = 0;
 	try { *_pSession << "INSERT INTO Person VALUES (?,?,?,?)", use(lastName), use(firstName), use(address), use(birthday), now; }
 	catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
@@ -1380,15 +1388,31 @@ void SQLExecutor::timestamp()
 	catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
 	catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
 	assertTrue (count == 1);
-	
+
 	DateTime bd;
 	assertTrue (bd != birthday);
 	try { *_pSession << "SELECT Birthday FROM Person", into(bd), now; }
 	catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
 	catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
 	assertTrue (bd == birthday);
-	
+
 	std::cout << std::endl << RecordSet(*_pSession, "SELECT * FROM Person");
+
+	Statement stmt(*_pSession);
+	stmt <<  "INSERT INTO Person VALUES (?,?,?,?)", use(lastName), use(firstName), use(address), use(birthday);
+	lastName = "Ridley";
+	firstName = "Sam";
+	birthday = DateTime(1982, 6, 13, 5, 30, 10, 0, 0);
+	try { stmt.execute(); }
+	catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
+	catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
+
+	try { *_pSession << "SELECT Birthday FROM Person WHERE LastName = 'Ridley'", into(bd), now; }
+	catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
+	catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
+	assertTrue (bd == birthday);
+
+
 }
 
 
@@ -1848,9 +1872,11 @@ void SQLExecutor::sessionTransaction(const std::string& connect)
 	catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
 	catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
 	assertTrue (0 == count);
-	assertTrue (!_pSession->isTransaction());
 
-	_pSession->begin();
+	if (_pSession->impl()->shouldParse()) {
+		assertTrue (!_pSession->isTransaction());
+		_pSession->begin();
+	}
 	try { (*_pSession) << "INSERT INTO Person VALUES (?,?,?,?)", use(lastNames), use(firstNames), use(addresses), use(ages), now; }
 	catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
 	catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
@@ -1934,11 +1960,13 @@ void SQLExecutor::transaction(const std::string& connect)
 	}
 	assertTrue (!_pSession->isTransaction());
 
-	try { (*_pSession) << "SELECT count(*) FROM Person", into(count), now; }
-	catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
-	catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
-	assertTrue (0 == count);
-	assertTrue (!_pSession->isTransaction());
+	if (_pSession->impl()->shouldParse()) {
+		try { (*_pSession) << "SELECT count(*) FROM Person", into(count), now; }
+		catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
+		catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
+		assertTrue (0 == count);
+		assertTrue (!_pSession->isTransaction());
+	}
 
 	{
 		Transaction trans((*_pSession));
@@ -1963,7 +1991,8 @@ void SQLExecutor::transaction(const std::string& connect)
 	catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
 	assertTrue (2 == count);
 
-	_pSession->begin();
+	if (_pSession->impl()->shouldParse())
+		_pSession->begin();
 	try { (*_pSession) << "DELETE FROM Person", now; }
 	catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
 	catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
@@ -1986,6 +2015,8 @@ void SQLExecutor::transaction(const std::string& connect)
 	sql.push_back(sql1);
 	sql.push_back(sql2);
 
+	assertTrue (!_pSession->isTransaction());
+
 	Transaction trans((*_pSession));
 
 	trans.execute(sql1, false);
@@ -2036,6 +2067,8 @@ void SQLExecutor::reconnect()
 	int count = 0;
 	std::string result;
 
+	_pSession->setFeature("autoCommit", true);
+
 	try { (*_pSession) << "INSERT INTO Person VALUES (?,?,?,?)", use(lastName), use(firstName), use(address), use(age), now;  }
 	catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
 	catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
@@ -2065,3 +2098,43 @@ void SQLExecutor::reconnect()
 	assertTrue (count == age);
 	assertTrue (_pSession->isConnected());
 }
+
+
+void SQLExecutor::sessionPoolAndUnicode(const std::string& connString)
+{
+	std::string funct = "unicode()";
+	std::string text = "ěščřžťďůň";
+	std::string text2;
+
+	// Test uses session from SessionPool instead of _pSession to prove session
+	// obtained and returned into pool is valid.
+
+	// Min/Max 1 session - ensures that when get() is called, same session should be returned
+	Poco::SharedPtr<Poco::Data::SessionPool> sp = new Poco::Data::SessionPool(MySQL::Connector::KEY, connString, 1, 1);
+
+	{
+		Poco::Data::Session session = sp->get();
+		try { session << "INSERT INTO Strings VALUES (?)", use(text), now; }
+		catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
+		catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
+	} // parentheses to ensure session is returned into pool
+
+	Poco::Data::Session session2 = sp->get();
+	try { session2 << "SELECT str FROM Strings", into(text2), now; }
+	catch(ConnectionException& ce){ std::cout << ce.displayText() << std::endl; fail (funct); }
+	catch(StatementException& se){ std::cout << se.displayText() << std::endl; fail (funct); }
+
+	assertTrue (text == text2);
+}
+
+
+void SQLExecutor::stdOptional()
+{
+	_dataExecutor.stdOptional();
+}
+
+
+void SQLExecutor::stdTupleWithOptional()
+{
+	_dataExecutor.stdTupleWithOptional();
+}
