@@ -63,7 +63,7 @@ void FileStreamBuf::open(const std::string& path, std::ios::openmode mode)
 
 	std::wstring utf16Path;
 	FileImpl::convertPath(path, utf16Path);
-	_handle = ::CreateFileW(utf16Path.c_str(), access, shareMode, NULL, creationDisp, flags, NULL);
+	_handle = ::CreateFileW(utf16Path.c_str(), access, shareMode, nullptr, creationDisp, flags, nullptr);
 
 	if (_handle == INVALID_HANDLE_VALUE)
 		File::handleLastError(_path);
@@ -89,7 +89,7 @@ void FileStreamBuf::openHandle(NativeHandle handle, std::ios::openmode mode)
 }
 
 
-int FileStreamBuf::readFromDevice(char* buffer, std::streamsize length)
+std::streamsize FileStreamBuf::readFromDevice(char* buffer, std::streamsize length)
 {
 	if (INVALID_HANDLE_VALUE == _handle || !(getMode() & std::ios::in))
 		return -1;
@@ -98,7 +98,7 @@ int FileStreamBuf::readFromDevice(char* buffer, std::streamsize length)
 		sync();
 
 	DWORD bytesRead(0);
-	BOOL rc = ::ReadFile(_handle, buffer, static_cast<DWORD>(length), &bytesRead, NULL);
+	BOOL rc = ::ReadFile(_handle, buffer, static_cast<DWORD>(length), &bytesRead, nullptr);
 	if (rc == 0)
 	{
 		if (::GetLastError() == ERROR_BROKEN_PIPE)
@@ -111,11 +111,11 @@ int FileStreamBuf::readFromDevice(char* buffer, std::streamsize length)
 
 	_pos += bytesRead;
 
-	return static_cast<int>(bytesRead);
+	return static_cast<std::streamsize>(bytesRead);
 }
 
 
-int FileStreamBuf::writeToDevice(const char* buffer, std::streamsize length)
+std::streamsize FileStreamBuf::writeToDevice(const char* buffer, std::streamsize length)
 {
 	if (INVALID_HANDLE_VALUE == _handle || !(getMode() & std::ios::out))
 		return -1;
@@ -131,13 +131,13 @@ int FileStreamBuf::writeToDevice(const char* buffer, std::streamsize length)
 	}
 
 	DWORD bytesWritten(0);
-	BOOL rc = ::WriteFile(_handle, buffer, static_cast<DWORD>(length), &bytesWritten, NULL);
+	BOOL rc = ::WriteFile(_handle, buffer, static_cast<DWORD>(length), &bytesWritten, nullptr);
 	if (rc == 0)
 		File::handleLastError(_path);
 
 	_pos += bytesWritten;
 
-	return static_cast<int>(bytesWritten);
+	return static_cast<std::streamsize>(bytesWritten);
 }
 
 
