@@ -26,12 +26,8 @@
 #if defined(POCO_OS_FAMILY_UNIX)
 #include "Path_UNIX.cpp"
 #elif defined(POCO_OS_FAMILY_WINDOWS)
-#if defined(_WIN32_WCE)
-#include "Path_WINCE.cpp"
-#else
 #include "Path_WIN32U.cpp"
 #endif
-#endif
 
 
 namespace Poco {
@@ -302,14 +298,14 @@ bool Path::tryParse(const std::string& path, Style style)
 
 Path& Path::parseDirectory(const std::string& path)
 {
-	assign(path);
+	assign(addDirectorySeparator(path));
 	return makeDirectory();
 }
 
 
 Path& Path::parseDirectory(const std::string& path, Style style)
 {
-	assign(path, style);
+	assign(addDirectorySeparator(path, style), style);
 	return makeDirectory();
 }
 
@@ -458,9 +454,9 @@ Path& Path::setDevice(const std::string& device)
 
 const std::string& Path::directory(int n) const
 {
-	poco_assert (0 <= n && n <= _dirs.size());
+	poco_assert (0 <= n && static_cast<std::size_t>(n) <= _dirs.size());
 
-	if (n < _dirs.size())
+	if (static_cast<std::size_t>(n) < _dirs.size())
 		return _dirs[n];
 	else
 		return _name;
@@ -469,9 +465,9 @@ const std::string& Path::directory(int n) const
 
 const std::string& Path::operator [] (int n) const
 {
-	poco_assert (0 <= n && n <= _dirs.size());
+	poco_assert (0 <= n && static_cast<std::size_t>(n) <= _dirs.size());
 
-	if (n < _dirs.size())
+	if (static_cast<std::size_t>(n) < _dirs.size())
 		return _dirs[n];
 	else
 		return _name;
@@ -537,7 +533,7 @@ Path& Path::setBaseName(const std::string& name)
 std::string Path::getBaseName() const
 {
 	std::string::size_type pos = _name.rfind('.');
-	if (pos != std::string::npos)
+	if (pos != std::string::npos && pos != 0)
 		return _name.substr(0, pos);
 	else
 		return _name;
@@ -559,7 +555,7 @@ Path& Path::setExtension(const std::string& extension)
 std::string Path::getExtension() const
 {
 	std::string::size_type pos = _name.rfind('.');
-	if (pos != std::string::npos)
+	if (pos != std::string::npos && pos != 0)
 		return _name.substr(pos + 1);
 	else
 		return std::string();
@@ -578,6 +574,55 @@ Path& Path::clear()
 }
 
 
+std::string Path::addDirectorySeparator(const std::string& path)
+{
+	poco_assert(!path.empty());
+
+	if (path.back() != separator())
+	{
+		return path + separator();
+	}
+	return path;
+}
+
+
+std::string Path::addDirectorySeparator(const std::string& path, Style style)
+{
+	poco_assert(!path.empty());
+
+	char ch = '\0';
+	switch (style)
+	{
+	case PATH_UNIX:
+		ch = '/';
+		break;
+	case PATH_WINDOWS:
+		ch = '\\';
+		break;
+	case PATH_VMS:
+		ch = '.';
+		break;
+	case PATH_NATIVE:
+		ch = separator();
+		break;
+	default:
+		poco_bugcheck();
+	}
+
+	if (path.back() != ch)
+	{
+		return path + ch;
+	}
+	return path;
+}
+
+
+std::string Path::self()
+{
+	return PathImpl::selfImpl();
+}
+
+
 std::string Path::current()
 {
 	return PathImpl::currentImpl();
@@ -776,8 +821,14 @@ void Path::parseWindows(const std::string& path)
 				_absolute = true;
 				_device += d;
 				++it;
-				if (it == end || (*it != '\\' && *it != '/')) throw PathSyntaxException(path);
-				++it;
+				if (it != end)
+				{
+					if ((*it != '\\' && *it != '/'))
+					{
+						throw PathSyntaxException(path);
+					}
+					++it;
+				}
 			}
 			else --it;
 		}
@@ -878,7 +929,7 @@ void Path::parseVMS(const std::string& path)
 							if (!_absolute) throw PathSyntaxException(path);
 							++it;
 							if (it != end && *it == '.') throw PathSyntaxException(path);
-							int d = int(_dirs.size());
+							auto d = _dirs.size();
 							while (it != end && *it != ']')
 							{
 								name.clear();
@@ -935,7 +986,9 @@ void Path::parseGuess(const std::string& path)
 			case '\\': hasBackslash = true; break;
 			case '/':  hasSlash = true; break;
 			case '[':  hasOpenBracket = true;
+				[[fallthrough]];
 			case ']':  hasClosBracket = hasOpenBracket;
+				[[fallthrough]];
 			case ';':  semiIt = it; break;
 			}
 		}
@@ -1067,11 +1120,11 @@ std::string Path::transcode(const std::string& path)
 #if defined(_WIN32)
 	std::wstring uniPath;
 	UnicodeConverter::toUTF16(path, uniPath);
-	DWORD len = WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, uniPath.c_str(), static_cast<int>(uniPath.length()), NULL, 0, NULL, NULL);
+	DWORD len = WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, uniPath.c_str(), static_cast<int>(uniPath.length()), nullptr, 0, nullptr, nullptr);
 	if (len > 0)
 	{
 		Buffer<char> buffer(len);
-		DWORD rc = WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, uniPath.c_str(), static_cast<int>(uniPath.length()), buffer.begin(), static_cast<int>(buffer.size()), NULL, NULL);
+		DWORD rc = WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, uniPath.c_str(), static_cast<int>(uniPath.length()), buffer.begin(), static_cast<int>(buffer.size()), nullptr, nullptr);
 		if (rc)
 		{
 			return std::string(buffer.begin(), buffer.size());
