@@ -69,7 +69,7 @@ std::istream* HTTPSStreamFactory::open(const URI& uri)
 
 	URI resolvedURI(uri);
 	URI proxyUri;
-	HTTPClientSession* pSession = 0;
+	HTTPClientSession* pSession = nullptr;
 	HTTPResponse res;
 	try
 	{
@@ -118,8 +118,8 @@ std::istream* HTTPSStreamFactory::open(const URI& uri)
 			pSession->sendRequest(req);
 			std::istream& rs = pSession->receiveResponse(res);
 			bool moved = (res.getStatus() == HTTPResponse::HTTP_MOVED_PERMANENTLY ||
-			              res.getStatus() == HTTPResponse::HTTP_FOUND ||
-			              res.getStatus() == HTTPResponse::HTTP_SEE_OTHER ||
+						  res.getStatus() == HTTPResponse::HTTP_FOUND ||
+						  res.getStatus() == HTTPResponse::HTTP_SEE_OTHER ||
 						  res.getStatus() == HTTPResponse::HTTP_TEMPORARY_REDIRECT);
 			if (moved)
 			{
@@ -130,7 +130,7 @@ std::istream* HTTPSStreamFactory::open(const URI& uri)
 					authorize = false;
 				}
 				delete pSession;
-				pSession = 0;
+				pSession = nullptr;
 				++redirects;
 				retry = true;
 			}
@@ -146,7 +146,7 @@ std::istream* HTTPSStreamFactory::open(const URI& uri)
 				// single request via the proxy. 305 responses MUST only be generated by origin servers.
 				// only use for one single request!
 				proxyUri.resolve(res.get("Location"));
-				delete pSession; pSession = 0;
+				delete pSession; pSession = nullptr;
 				retry = true; // only allow useproxy once
 			}
 			else if (res.getStatus() == HTTPResponse::HTTP_UNAUTHORIZED && !authorize)
