@@ -33,6 +33,8 @@ namespace Net {
 
 const std::string SSLManager::CFG_CERT_NAME("certificateName");
 const std::string SSLManager::VAL_CERT_NAME("");
+const std::string SSLManager::CFG_CERT_HASH("certificateHash");
+const std::string SSLManager::VAL_CERT_HASH("");
 const std::string SSLManager::CFG_CERT_PATH("certificatePath");
 const std::string SSLManager::VAL_CERT_PATH("");
 const std::string SSLManager::CFG_CERT_STORE("certificateStore");
@@ -60,7 +62,7 @@ const std::string SSLManager::CFG_REQUIRE_TLSV1_3("requireTLSv1_3");
 
 
 SSLManager::SSLManager():
-	_hSecurityModule(0)
+	_hSecurityModule(nullptr)
 {
 	loadSecurityLibrary();
 }
@@ -188,7 +190,8 @@ void SSLManager::initDefaultContext(bool server)
 
 	const std::string prefix = server ? CFG_SERVER_PREFIX : CFG_CLIENT_PREFIX;
 	Poco::Util::AbstractConfiguration& config = appConfig();
-	std::string certName = config.getString(prefix + CFG_CERT_NAME, VAL_CERT_NAME);
+	std::string certInfo = config.getString(prefix + CFG_CERT_NAME, VAL_CERT_NAME);
+	std::string certHash = config.getString(prefix + CFG_CERT_HASH, VAL_CERT_HASH);
 	std::string certPath = config.getString(prefix + CFG_CERT_PATH, VAL_CERT_PATH);
 	std::string certStore = config.getString(prefix + CFG_CERT_STORE, VAL_CERT_STORE);
 
@@ -218,7 +221,12 @@ void SSLManager::initDefaultContext(bool server)
 	if (!certPath.empty())
 	{
 		options |= Context::OPT_LOAD_CERT_FROM_FILE;
-		certName = certPath;
+		certInfo = certPath;
+	}
+	if (certInfo.empty() && !certHash.empty())
+	{
+		options |= Context::OPT_USE_CERT_HASH;
+		certInfo = certHash;
 	}
 
 	Context::Usage usage;
@@ -234,7 +242,7 @@ void SSLManager::initDefaultContext(bool server)
 			usage = Context::TLSV1_SERVER_USE;
 		else
 			usage = Context::SERVER_USE;
-		_ptrDefaultServerContext = new Context(usage, certName, verMode, options, certStore);
+		_ptrDefaultServerContext = new Context(usage, certInfo, verMode, options, certStore);
 	}
 	else
 	{
@@ -248,7 +256,7 @@ void SSLManager::initDefaultContext(bool server)
 			usage = Context::TLSV1_CLIENT_USE;
 		else
 			usage = Context::CLIENT_USE;
-		_ptrDefaultClientContext = new Context(usage, certName, verMode, options, certStore);
+		_ptrDefaultClientContext = new Context(usage, certInfo, verMode, options, certStore);
 	}
 }
 
@@ -270,7 +278,7 @@ void SSLManager::initPassphraseHandler(bool server)
 
 	std::string className(config.getString(prefix + CFG_DELEGATE_HANDLER, VAL_DELEGATE_HANDLER));
 
-	const PrivateKeyFactory* pFactory = 0;
+	const PrivateKeyFactory* pFactory = nullptr;
 	if (privateKeyFactoryMgr().hasFactory(className))
 	{
 		pFactory = privateKeyFactoryMgr().getFactory(className);
@@ -297,7 +305,7 @@ void SSLManager::initCertificateHandler(bool server)
 
 	std::string className(config.getString(prefix + CFG_CERTIFICATE_HANDLER, VAL_CERTIFICATE_HANDLER));
 
-	const CertificateHandlerFactory* pFactory = 0;
+	const CertificateHandlerFactory* pFactory = nullptr;
 	if (certificateHandlerFactoryMgr().hasFactory(className))
 	{
 		pFactory = certificateHandlerFactoryMgr().getFactory(className);
@@ -318,12 +326,12 @@ void SSLManager::shutdown()
 {
 	ClientVerificationError.clear();
 	ServerVerificationError.clear();
-	_ptrServerPassphraseHandler  = 0;
-	_ptrServerCertificateHandler = 0;
-	_ptrDefaultServerContext     = 0;
-	_ptrClientPassphraseHandler  = 0;
-	_ptrClientCertificateHandler = 0;
-	_ptrDefaultClientContext     = 0;
+	_ptrServerPassphraseHandler  = nullptr;
+	_ptrServerCertificateHandler = nullptr;
+	_ptrDefaultServerContext     = nullptr;
+	_ptrClientPassphraseHandler  = nullptr;
+	_ptrClientCertificateHandler = nullptr;
+	_ptrDefaultClientContext     = nullptr;
 
 	unloadSecurityLibrary();
 }
@@ -363,7 +371,7 @@ void SSLManager::loadSecurityLibrary()
 	//
 
 	_hSecurityModule = LoadLibraryW(dllPath.c_str());
-	if(_hSecurityModule == 0)
+	if(_hSecurityModule == nullptr)
 	{
 		throw Poco::SystemException("Failed to load security DLL");
 	}
@@ -373,7 +381,7 @@ void SSLManager::loadSecurityLibrary()
 	if (!pInitSecurityInterface)
 	{
 		FreeLibrary(_hSecurityModule);
-		_hSecurityModule = 0;
+		_hSecurityModule = nullptr;
 		throw Poco::SystemException("Failed to initialize security DLL (no init function)");
 	}
 
@@ -381,7 +389,7 @@ void SSLManager::loadSecurityLibrary()
 	if (!pSecurityFunc)
 	{
 		FreeLibrary(_hSecurityModule);
-		_hSecurityModule = 0;
+		_hSecurityModule = nullptr;
 		throw Poco::SystemException("Failed to initialize security DLL (no function table)");
 	}
 
@@ -394,7 +402,7 @@ void SSLManager::unloadSecurityLibrary()
 	if (_hSecurityModule)
 	{
 		FreeLibrary(_hSecurityModule);
-		_hSecurityModule = 0;
+		_hSecurityModule = nullptr;
 	}
 }
 
