@@ -8,7 +8,6 @@
 
 
 #include "Poco/PDF/PDF.h"
-#include "Poco/PDF/Font.h"
 #include "Poco/Dynamic/Var.h"
 
 
@@ -62,30 +61,6 @@ private:
 };
 
 
-//
-// inlines
-//
-
-inline AttributedString& AttributedString::operator=(const std::string& content)
-{
-	_content = content;
-	return *this;
-}
-
-
-inline AttributedString& AttributedString::operator=(const char* content)
-{
-	_content = content;
-	return *this;
-}
-
-
-inline AttributedString::operator const std::string&()
-{
-	return _content;
-}
-
-
 } } // namespace Poco::PDF
 
 
