@@ -33,7 +33,7 @@ public:
 	Image(HPDF_Doc* pPDF, const HPDF_Image& resource, const std::string& name = "");
 		/// Creates the image.
 
-	~Image();
+	~Image() override;
 		/// Destroys the image.
 
 	Point size() const;
@@ -62,56 +62,6 @@ public:
 };
 
 
-//
-// inlines
-//
-
-inline Point Image::size() const
-{
-	return HPDF_Image_GetSize(handle());
-}
-
-
-inline float Image::width() const
-{
-	return static_cast<float>(HPDF_Image_GetWidth(handle()));
-}
-
-
-inline float Image::height() const
-{
-	return static_cast<float>(HPDF_Image_GetHeight(handle()));
-}
-
-
-inline Poco::UInt32 Image::bitsPerColor() const
-{
-	return HPDF_Image_GetBitsPerComponent(handle());
-}
-
-
-inline std::string Image::colorSpace() const
-{
-	return HPDF_Image_GetColorSpace(handle());
-}
-
-
-inline void Image::colorMask(Poco::UInt32 redMin,
-	Poco::UInt32 redMax,
-	Poco::UInt32 greenMin,
-	Poco::UInt32 greenMax,
-	Poco::UInt32 blueMin,
-	Poco::UInt32 blueMax)
-{
-	HPDF_Image_SetColorMask(handle(),
-		redMin,
-		redMax,
-		greenMin,
-		greenMax,
-		blueMin,
-		blueMax);
-}
-
 
 } } // namespace Poco::PDF
 
