@@ -13,8 +13,7 @@
 
 
 #include "Poco/PDF/LinkAnnotation.h"
-#include "Poco/PDF/PDFException.h"
-
+#include <hpdf.h>
 
 namespace Poco {
 namespace PDF {
@@ -27,10 +26,19 @@ LinkAnnotation::LinkAnnotation(HPDF_Doc* pPDF,
 {
 }
 
+LinkAnnotation::~LinkAnnotation() = default;
 
-LinkAnnotation::~LinkAnnotation()
+void LinkAnnotation::setHighlight(Highlight mode)
 {
+	HPDF_LinkAnnot_SetHighlightMode(handle(),
+									static_cast<HPDF_AnnotHighlightMode>(mode));
+}
+
+void LinkAnnotation::setBorderStyle(float width, Poco::UInt32 dashOn, Poco::UInt32 dashOff)
+{
+	HPDF_LinkAnnot_SetBorderStyle(handle(), width, dashOn, dashOff);
 }
 
 
+
 } } // namespace Poco::PDF
