@@ -13,8 +13,8 @@
 
 
 #include "Poco/PDF/TextAnnotation.h"
-#include "Poco/PDF/PDFException.h"
 
+#include <hpdf.h>
 
 namespace Poco {
 namespace PDF {
@@ -35,4 +35,22 @@ TextAnnotation::~TextAnnotation()
 }
 
 
+void TextAnnotation::open()
+{
+	HPDF_TextAnnot_SetOpened(handle(), HPDF_TRUE);
+}
+
+
+void TextAnnotation::close()
+{
+	HPDF_TextAnnot_SetOpened(handle(), HPDF_FALSE);
+}
+
+
+void TextAnnotation::icon(IconType iconType)
+{
+	HPDF_TextAnnot_SetIcon(handle(), static_cast<HPDF_AnnotIcon>(iconType));
+}
+
+
 } } // namespace Poco::PDF
